@@ -33,10 +33,12 @@ class Device(BaseRegistry):
     _active = False
     _output_thread = None
     _active_effect = None
+    _fadeout_effect = None
 
     def __init__(self, ledfx, config):
         self._ledfx = ledfx
         self._config = config
+        self.fade_timer = 0 # the multiplier to fade in/out of an effect. -ve values mean fading in, +ve mean fading out
 
     def __del__(self):
         if self._active:
@@ -47,8 +49,13 @@ class Device(BaseRegistry):
         pass
 
     def set_effect(self, effect, start_pixel = None, end_pixel = None):
+        self.fade_duration = self._config['refresh_rate'] * self._ledfx.config['fade']
+        self.fade_timer = self.fade_duration
+
         if self._active_effect != None:
-            self._active_effect.deactivate()
+            self._fadeout_effect = self._active_effect
+            self._ledfx.loop.call_later(self._ledfx.config['fade'], self.clear_fadeout_effect)
+
 
         self._active_effect = effect
         self._active_effect.activate(self.pixel_count)
@@ -57,16 +64,27 @@ class Device(BaseRegistry):
             self.activate()
 
     def clear_effect(self):
+        self.fade_duration = self._config['refresh_rate'] * self._ledfx.config['fade']
+        self.fade_timer = -self.fade_duration
+
+        self._ledfx.loop.call_later(self._ledfx.config['fade'], self.clear_frame)
+
+    def clear_fadeout_effect(self):
+        if self._fadeout_effect is not None:
+            self._fadeout_effect.deactivate()
+        self._fadeout_effect = None
+
+    def clear_frame(self):
         if self._active_effect != None:
             self._active_effect.deactivate()
             self._active_effect = None
         
         if self._active:
-            # Clear all the pixel data before deactiving the device
-            assembled_frame = np.zeros((self.pixel_count, 3))
-            self.flush(assembled_frame)
+            # Clear all the pixel data before deactivating the device
+            self.assembled_frame = np.zeros((self.pixel_count, 3))
+            self.flush(self.assembled_frame)
             self._ledfx.events.fire_event(DeviceUpdateEvent(
-                self.id, assembled_frame))
+                self.id, self.assembled_frame))
 
             self.deactivate()
 
@@ -76,14 +94,14 @@ class Device(BaseRegistry):
 
     def process_active_effect(self):
         # Assemble the frame if necessary, if nothing changed just sleep
-        assembled_frame = self.assemble_frame()
-        if assembled_frame is not None:
+        self.assembled_frame = self.assemble_frame()
+        if self.assembled_frame is not None:
             if not self._config['preview_only']:
-                self.flush(assembled_frame)
+                self.flush(self.assembled_frame)
 
             def trigger_device_update_event(): 
                 self._ledfx.events.fire_event(DeviceUpdateEvent(
-                    self.id, assembled_frame))
+                    self.id, self.assembled_frame))
             self._ledfx.loop.call_soon_threadsafe(trigger_device_update_event)
 
     def thread_function(self):
@@ -115,11 +133,41 @@ class Device(BaseRegistry):
         """
         frame = None
         if self._active_effect._dirty:
+            # Get and process active effect frame
             frame = np.clip(self._active_effect.pixels * self._config['max_brightness'], 0, 255)
             if self._config['center_offset']:
                 frame = np.roll(frame, self._config['center_offset'], axis = 0)
-
             self._active_effect._dirty = self._config['force_refresh']
+            
+            # Handle fading effect in/out if just turned on or off
+            if self.fade_timer == 0:
+                pass    
+            elif self.fade_timer > 0:
+                # if +ve fade timer, fade in the effect
+                frame *= 1 - (self.fade_timer / self.fade_duration)
+                self.fade_timer -= 1
+            elif self.fade_timer < 0:
+                # if -ve fade timer, fade out the effect
+                frame *= -self.fade_timer / self.fade_duration
+                self.fade_timer += 1
+        
+        # This part handles blending two effects together
+        fadeout_frame = None
+        if self._fadeout_effect:
+            if self._fadeout_effect._dirty:
+                # Get and process fadeout effect frame
+                fadeout_frame = np.clip(self._fadeout_effect.pixels * self._config['max_brightness'], 0, 255)
+                if self._config['center_offset']:
+                    fadeout_frame = np.roll(fadeout_frame, self._config['center_offset'], axis = 0)
+                self._fadeout_effect._dirty = self._config['force_refresh']
+
+                # handle fading out the fadeout frame
+                if self.fade_timer:
+                    fadeout_frame *= self.fade_timer / self.fade_duration
+
+        # Blend both frames together
+        if (fadeout_frame is not None) and (frame is not None):
+            frame += fadeout_frame
 
         return frame
 
@@ -191,7 +239,7 @@ class Devices(RegistryLoader):
 
     def clear_all_effects(self):
         for device in self.values():
-            device.clear_effect()
+            device.clear_frame()
 
     def get_device(self, device_id):
         for device in self.values():
@@ -199,16 +247,14 @@ class Devices(RegistryLoader):
                 return device
         return None
 
-    def find_wled_devices(self):
+    async def find_wled_devices(self):
         #Scan the LAN network that match WLED using zeroconf - Multicast DNS Service Discovery Library
         _LOGGER.info("Scanning for WLED devices...")
         zeroconf_obj = zeroconf.Zeroconf()
         listener = MyListener(self._ledfx)
         browser = zeroconf.ServiceBrowser(zeroconf_obj, "_wled._tcp.local.", listener)
         try:
-            # TODO make this non blocking
-            # await asyncio.sleep(1.5)
-            time.sleep(1.5)
+            await asyncio.sleep(10)
         finally:
             _LOGGER.info("Scan Finished")
             zeroconf_obj.close()
@@ -219,7 +265,7 @@ class MyListener:
         self._ledfx = _ledfx
 
     def remove_service(self, zeroconf_obj, type, name):
-        print(f"Service {name} removed")
+        _LOGGER.info(f"Service {name} removed")
 
     def add_service(self, zeroconf_obj, type, name):
         # DMX universe_size
