@@ -7,6 +7,13 @@ from ledfx.api import RestApi
 import numpy as np
 import json
 import ledfx_frontend
+import os
+import sys
+
+try:
+    base_path = sys._MEIPASS
+except:
+    base_path = os.path.abspath(".")
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -16,9 +23,10 @@ class HttpServer(object):
 
         self.app = web.Application(loop=ledfx.loop)
         self.api = RestApi(ledfx)
+        templates_path = os.path.abspath(os.path.dirname(ledfx_frontend.__file__))
         aiohttp_jinja2.setup(
             self.app,
-            loader=jinja2.PackageLoader('ledfx_frontend', '.'))
+            loader=jinja2.FileSystemLoader(templates_path))
         self.register_routes()
 
         self._ledfx = ledfx
